(function(U,V,W,X,K,Y,O,r,P,Q,R,N,Z,S,$,H,M,aa,z,ba,ca,da,ea,fa,ga,ha,ia,ja,ka,la,C,ma,na,oa,pa,qa,ra,sa,ta,ua,va,wa,xa,ya,za,Aa,Ba,Ca,Da,Ea,Fa,Ga,Ha,Ia,Ja,Ka,La,Ma,m){O("JS");P(null,"JS.MathExt","java.lang.Float $.Number java.util.Date $.Hashtable $.Random JU.AU $.BS $.CU $.Lst $.M4 $.Measure $.OC $.P3 $.P4 $.PT $.Quat $.Rdr $.SB $.V3 J.api.Interface J.atomdata.RadiusData J.bspt.PointIterator J.c.VDW J.i18n.GT JM.BondSet $.Measurement JS.SV $.ScriptParam $.T JU.BSUtil $.Escape $.JmolMolecule $.Logger $.Parser $.Point3fi $.SimpleUnitCell JV.FileManager $.JC $.Viewer".split(" "),
function(){c$=R(function(){this.pm=this.rand=this.e=this.vwr=null;Q(this,arguments)},JS,"MathExt");S(c$,function(){});m(c$,"init",function(d){this.e=d;this.vwr=this.e.vwr;return this},"~O");m(c$,"evaluate",function(d,a,b,c){switch(c){case 134218250:case 134218242:case 134218245:case 134217749:case 134218244:case 134218246:return this.evaluateMath(d,b,c);case 1275069441:case 1275068928:case 1275068929:case 1275068930:case 1275068931:case 1275335685:case 1275334681:return this.evaluateList(d,a.intValue,
b);case 268435520:0==b.length&&(d.wasX=!1);case 1275068418:return this.evaluateArray(d,b,1275068418==c&&268435665==a.tok);case 134217731:case 134221850:return this.evaluateQuaternion(d,b,c);case 1275068420:return this.evaluateBin(d,b);case 134221829:return this.evaluateCache(d,b);case 1275068934:case 1275068935:return this.evaluateRowCol(d,b,c);case 1765808134:return this.evaluateColor(d,b);case 134221831:return this.evaluateCompare(d,b);case 1228931587:case 134217736:case 1275203608:return this.evaluateConnected(d,
b,c,a.intValue);case 1814695966:return this.evaluateUnitCell(d,b,268435665==a.tok);case 134353926:return this.evaluateContact(d,b);case 134221834:return this.evaluateData(d,b);case 1275069444:case 1275069442:return this.evaluateDotDist(d,b,c,a.intValue);case 1275069443:if(268435665==a.tok)return this.evaluateDotDist(d,b,c,a.intValue);case 134217729:case 1745489939:return this.evaluateMeasure(d,b,a.tok);case 1228935687:case 134222849:return this.evaluateLoad(d,b,1228935687==c);case 1275068427:return this.evaluateFind(d,
b);case 1287653388:case 1825200146:return this.evaluateFormat(d,a.intValue,b,1825200146==c);case 134320141:return this.evaluateUserFunction(d,a.value,b,a.intValue,268435665==a.tok);case 1275068449:case 1275068437:case 1275082245:case 1275072526:return this.evaluateGetProperty(d,b,c,268435665==a.tok);case 136314895:return this.evaluateHelix(d,b);case 134219265:case 134217750:case 134217763:return this.evaluatePlane(d,b,c);case 134218759:case 134238732:case 134222850:case 134222350:return this.evaluateScript(d,
b,c);case 1275069446:case 1275069447:case 1275068932:return this.evaluateString(d,a.intValue,b);case 134217751:return this.evaluatePoint(d,b);case 134217762:return this.evaluatePointGroup(d,b);case 134256131:return this.evaluatePrompt(d,b);case 134219268:return this.evaluateRandom(d,b);case 1275068432:return this.evaluateIn(d,b);case 1275072532:return this.evaluateModulation(d,b);case 1275068443:return this.evaluateReplace(d,b);case 134218756:case 134218757:case 1237320707:return this.evaluateSubstructure(d,
b,c,268435665==a.tok);case 1275068444:case 1275068425:return this.evaluateSort(d,b,c);case 134217764:return this.evaluateSpacegroup(d,b);case 1296041986:return this.evaluateSymop(d,b,268435665==a.tok);case 1275068445:return this.evaluateTensor(d,b);case 134217759:return this.evaluateWithin(d,b);case 134221856:return this.evaluateWrite(d,b)}return!1},"JS.ScriptMathProcessor,JS.T,~A,~N");m(c$,"evaluateSpacegroup",function(d,a){var b=null;switch(a.length){case 0:return d.addXObj(this.vwr.getSymTemp().getSpaceGroupInfo(this.vwr.ms,
null,this.vwr.am.cmi,!0,null));case 2:b=JS.SV.flistValue(a[1],0),6>b.length&&(b=null);case 1:return d.addXObj(this.vwr.getSymTemp().getSpaceGroupInfo(this.vwr.ms,""+a[0].asString(),-2147483648,!0,b));default:return!1}},"JS.ScriptMathProcessor,~A");m(c$,"evaluatePointGroup",function(d,a){var b=null,c=null,e=NaN,g=NaN,f;switch(a.length){case 4:g=a[3].asFloat();case 3:e=a[2].asFloat();case 2:switch(a[1].tok){case 8:c=JS.SV.ptValue(a[1]);break;case 10:f=JS.SV.getBitSet(a[1],!1);c=f.nextSetBit(0);if(0>
c||c>=this.vwr.ms.ac||1!=f.cardinality())return!1;if(JS.SV.sValue(a[0]).equalsIgnoreCase("spaceGroup")){c=this.vwr.ms.generateCrystalClass(c,JU.P3.new3(NaN,NaN,NaN));b=Array(c.size());for(f=b.length;0<=--f;)b[f]=c.get(f);c=new JU.P3;2==a.length&&(e=0)}else c=this.vwr.ms.at[c]}if(null!=b)break;case 1:switch(a[0].tok){case 7:var h=a[0].getList(),b=Array(h.size());for(f=b.length;0<=--f;)b[f]=JS.SV.ptValue(h.get(f));break;case 10:f=JS.SV.getBitSet(a[0],!1);h=this.vwr.ms.getAtomPointVector(f);b=Array(h.size());
for(f=b.length;0<=--f;)b[f]=h.get(f);break;default:return!1}break;default:return!1}b=this.vwr.getSymTemp().setPointGroup(null,c,b,null,!1,Float.isNaN(e)?this.vwr.getFloat(570425382):e,Float.isNaN(g)?this.vwr.getFloat(570425384):g,!0);return d.addXMap(b.getPointGroupInfo(-1,null,!0,null,0,1))},"JS.ScriptMathProcessor,~A");m(c$,"evaluateUnitCell",function(d,a,b){b=b?JS.SV.getBitSet(d.getX(),!0):null;var c=(null==b?this.vwr.getAllAtoms():b).nextSetBit(0),e=a.length-1;b=1;switch(0>e?0:a[e].tok){case 2:case 3:b=
a[e].asFloat(),e--}var g=0>e?0:a[0].tok,f=null,h=null;switch(g){case 7:h=a[0].getList();break;case 4:var k=a[0].asString();if(0==k.indexOf("a=")){for(var f=Array(4),j=0;4>j;j++)f[j]=new JU.P3;JU.SimpleUnitCell.setOabc(k,null,f)}else if(0<=k.indexOf(","))return d.addXObj(this.vwr.getV0abc(k))}k=null;j=null!=h;if(null==f&&j&&4>h.size())return!1;var l=j?1:0;null==f&&(!j&&8!=g)&&(k=0>c?null:this.vwr.ms.getUnitCell(this.vwr.ms.at[c].mi),f=null==k?C(-1,[JU.P3.new3(0,0,0),JU.P3.new3(1,0,0),JU.P3.new3(0,
1,0),JU.P3.new3(0,0,1)]):k.getUnitCellVectors());if(null==f)if(f=Array(4),j)switch(h.size()){case 3:f[0]=new JU.P3;for(j=0;3>j;j++)f[j+1]=JU.P3.newP(JS.SV.ptValue(h.get(j)));break;case 4:for(j=0;4>j;j++)f[j]=JU.P3.newP(JS.SV.ptValue(h.get(j)));break;case 6:g=z(6,0);for(j=0;6>j;j++)g[j]=h.get(j).asFloat();JU.SimpleUnitCell.setOabc(null,g,f);break;default:return!1}else switch(f[0]=JS.SV.ptValue(a[0]),e){case 3:for(j=1;4>j;j++)(f[j]=JU.P3.newP(JS.SV.ptValue(a[j]))).sub(f[0]);break;case 1:if(h=a[1].getList(),
null!=h&&3==h.size()){for(j=0;3>j;j++)f[j+1]=JS.SV.ptValue(h.get(j));break}default:return!1}j=l<=e?a[l].asString():null;if((h="primitive".equalsIgnoreCase(j))||"conventional".equalsIgnoreCase(j)){a=++l>e?"":a[l].asString().toUpperCase();a.equals("BCC")?a="I":0==a.length&&(a=this.vwr.getSymmetryInfo(c,null,0,null,null,1073741994,null,0,-1,0));if(null==a||0==a.length)return!1;null==k&&(k=this.vwr.getSymTemp());c=this.vwr.getModelForAtomIndex(c).auxiliaryInfo.get("primitiveToCrystal");if(!k.toFromPrimitive(h,
a.charAt(0),f,c))return!1}else"reciprocal".equalsIgnoreCase(j)&&(f=JU.SimpleUnitCell.getReciprocal(f,null,b),b=1);if(1!=b)for(j=1;4>j;j++)f[j].scale(b);return d.addXObj(f)},"JS.ScriptMathProcessor,~A,~B");m(c$,"evaluateArray",function(d,a,b){if(b){b=d.getX();switch(1==a.length?b.tok:0){case 6:var c=new JU.Lst,e=a[0].asString(),g=b.getMap(),f=b.getKeys(!1);b=0;for(a=f.length;b<a;b++)if(null==g.get(f[b]).getMap())return!1;b=0;for(a=f.length;b<a;b++){var h=g.get(f[b]),h=h.getMap(),h=JS.SV.deepCopy(h,
!0,!1);h.put(e,JS.SV.newS(f[b]));c.addLast(JS.SV.newV(6,h))}return d.addXList(c);case 7:c=new java.util.Hashtable;e=b.getList();g=a[0].asString();b=0;for(a=e.size();b<a;b++)if(h=e.get(b).getMap(),null==h||null==h.get(g))return!1;b=0;for(a=e.size();b<a;b++)h=e.get(b),h=JS.SV.deepCopy(h.getMap(),!0,!1),f=h.remove(g),c.put(f.asString(),JS.SV.newV(6,h));return d.addXObj(c)}return!1}c=Array(a.length);for(b=c.length;0<=--b;)c[b]=JS.SV.newT(a[b]);return d.addXAV(c)},"JS.ScriptMathProcessor,~A,~B");m(c$,
"evaluateBin",function(d,a){var b=a.length;if(3>b||5<b)return!1;var c=d.getX(),e=13==c.tok;if(!e&&7!=c.tok)return d.addX(c);var g=JS.SV.fValue(a[0]),f=JS.SV.fValue(a[1]),h=JS.SV.fValue(a[2]),k=4<=b&&1073742335==a[b-1].tok,b=(5==b||4==b&&!k)&&1073742334!=a[3].tok?JS.SV.sValue(a[3]):null,j=null;if(e)c=c.value;else{e=c.getList();c=z(e.size(),0);null!=b&&(j=JU.AU.createArrayOfHashtable(e.size()));try{for(var l=e.size();0<=--l;)c[l]=JS.SV.fValue(null==b?e.get(l):(j[l]=e.get(l).getMap()).get(b))}catch(m){if(H(m,
Exception))return!1;throw m;}}for(var f=Math.max(K(Math.floor((f-g)/h+0.01)),1),e=M(f,0),p=c.length,l=0;l<p;l++){var n=K(Math.floor((c[l]-g)/h));if(!(0>n||n>=f))if(e[n]++,null!=b){var q=j[l];if(null!=q){q.put("_bin",JS.SV.newI(n));var y=g+h*n,u=y+h;q.put("_binMin",JS.SV.newF(0==n?-3.4028235E38:y));q.put("_binMax",JS.SV.newF(n==f-1?3.4028235E38:u))}}}if(k){k=new JU.Lst;for(l=0;l<f;l++)k.addLast(z(-1,[g+h*l,e[l]]));return d.addXList(k)}return d.addXAI(e)},"JS.ScriptMathProcessor,~A");m(c$,"evaluateCache",
function(d,a){return 0<a.length?!1:d.addXMap(this.vwr.fm.cacheList())},"JS.ScriptMathProcessor,~A");m(c$,"evaluateColor",function(d,a){var b=0<a.length?JS.SV.sValue(a[0]):"",c=b.startsWith("$");if(2==a.length&&b.equalsIgnoreCase("TOHSL"))return d.addXPt(JU.CU.rgbToHSL(JU.P3.newP(8==a[1].tok?JS.SV.ptValue(a[1]):JU.CU.colorPtFromString(a[1].asString())),!0));if(2==a.length&&b.equalsIgnoreCase("TORGB")){var e=JU.P3.newP(8==a[1].tok?JS.SV.ptValue(a[1]):JU.CU.colorPtFromString(a[1].asString()));return d.addXPt(8==
a[1].tok?JU.CU.hslToRGB(e):e)}if(4==a.length&&(1073742335==a[3].tok||1073742334==a[3].tok)){var e=JU.P3.newP(8==a[0].tok?JS.SV.ptValue(a[0]):JU.CU.colorPtFromString(a[0].asString())),g=JU.P3.newP(8==a[1].tok?JS.SV.ptValue(a[1]):JU.CU.colorPtFromString(a[1].asString())),f=1073742335==a[3].tok;f&&(e=JU.CU.rgbToHSL(e,!1),g=JU.CU.rgbToHSL(g,!1));var h=new JU.SB,g=JU.V3.newVsub(g,e),k=a[2].asInt();2>k&&(k=20);g.scale(1/(k-1));for(b=0;b<k;b++)h.append(JU.Escape.escapeColor(JU.CU.colorPtToFFRGB(f?JU.CU.hslToRGB(e):
e))),e.add(g);return d.addXStr(h.toString())}k=c?null:this.vwr.cm.getColorEncoder(b);if(!c&&null==k)return d.addXStr("");e=1<a.length?JS.SV.fValue(a[1]):3.4028235E38;g=2<a.length?JS.SV.fValue(a[2]):3.4028235E38;f=3<a.length?JS.SV.fValue(a[3]):3.4028235E38;h=3.4028235E38!=f||3.4028235E38!=e&&3.4028235E38==g;3.4028235E38==g&&0==b.length&&(f=e,g=this.vwr.getCurrentColorRange(),e=g[0],g=g[1]);if(c){k=b.substring(1);k=C(-1,[k,null]);if(!this.vwr.shm.getShapePropertyData(24,"colorEncoder",k))return d.addXStr("");
k=k[1]}else k.setRange(e,g,e>g);b=k.getColorKey();return h?d.addXPt(JU.CU.colorPtFromInt(k.getArgb(3.4028235E38==g?e:f),null)):d.addX(JS.SV.getVariableMap(b))},"JS.ScriptMathProcessor,~A");m(c$,"evaluateCompare",function(d,a){if(2>a.length||5<a.length)return!1;var b,c=JS.SV.sValue(a[a.length-1]),e=c.equalsIgnoreCase("stddev"),g=c.equalsIgnoreCase("ISOMER"),f=c.equalsIgnoreCase("BONDS"),h=!g&&a.length>(e?3:2),k=10==a[0].tok?a[0].value:null,j=10==a[1].tok?a[1].value:null,l=null==k?JS.SV.sValue(a[0]):
"",m=null==j?JS.SV.sValue(a[1]):"",p=new JU.M4;b=NaN;var n,q;try{if(h&&(null==k||null==j))return!1;if(f){if(4!=a.length)return!1;l=JS.SV.sValue(a[2]);h=l.equalsIgnoreCase("SMILES");try{h&&(l=this.vwr.getSmiles(k))}catch(y){if(H(y,Exception))this.e.evalError(y.getMessage(),null);else throw y;}var u=this.e.getSmilesExt().getFlexFitList(k,j,l,!h);return null==u?d.addXStr(""):d.addXAF(u)}if(g){if(3!=a.length)return!1;if(null==k&&null==j)return d.addXStr(this.vwr.getSmilesMatcher().getRelationship(l,m).toUpperCase());
var v=null==k?this.vwr.getSmilesMatcher().getMolecularFormula(l,!1):JU.JmolMolecule.getMolecularFormulaAtoms(this.vwr.ms.at,k,null,!1),T=null==j?this.vwr.getSmilesMatcher().getMolecularFormula(m,!1):JU.JmolMolecule.getMolecularFormulaAtoms(this.vwr.ms.at,j,null,!1);if(!v.equals(T))return d.addXStr("NONE");null!=k&&(l=this.e.getSmilesExt().getSmilesMatches("/strict///",null,k,null,1,!0,!1));var r;null==j?r=0<this.vwr.getSmilesMatcher().areEqual(m,l):(m=this.e.getSmilesExt().getSmilesMatches("/strict///",
null,j,null,1,!0,!1),r=0<=this.e.getSmilesExt().getSmilesMatches("/strict///"+l,null,j,null,1,!0,!1).nextSetBit(0));if(!r){n=l+m;if(0<=n.indexOf("/")||0<=n.indexOf("\\")||0<=n.indexOf("@")){if(0<=l.indexOf("@")&&(null!=j||0<=m.indexOf("@"))&&0>l.indexOf("@SP")){var t=l.toLowerCase().indexOf("invertstereo"),l=0<=t?"/strict/"+l.substring(0,t)+l.substring(t+12):"/invertstereo strict/"+l;if(r=null==j?0<this.vwr.getSmilesMatcher().areEqual(l,m):0<=this.e.getSmilesExt().getSmilesMatches(l,null,j,null,1,
!0,!1).nextSetBit(0))return d.addXStr("ENANTIOMERS")}if(null==j)r=0<this.vwr.getSmilesMatcher().areEqual("/nostereo/"+m,l);else{var s=this.e.getSmilesExt().getSmilesMatches("/nostereo/"+l,null,j,null,1,!0,!1);r=0<=s.nextSetBit(0)}if(r)return d.addXStr("DIASTEREOMERS")}return d.addXStr("CONSTITUTIONAL ISOMERS")}if(null==k||null==j)return d.addXStr("IDENTICAL");b=this.e.getSmilesExt().getSmilesCorrelation(k,j,l,null,null,null,null,!1,null,null,!1,1);return d.addXStr(0.2>b?"IDENTICAL":"IDENTICAL or CONFORMATIONAL ISOMERS (RMSD="+
b+")")}if(h){n=new JU.Lst;q=new JU.Lst;var c=JS.SV.sValue(a[2]),x=c.equalsIgnoreCase("MAP"),h=c.equalsIgnoreCase("SMILES"),w=x||c.equalsIgnoreCase("SMARTS");if(h||w)c=a.length>(e?4:3)?JS.SV.sValue(a[3]):null;var z="H".equalsIgnoreCase(c)||"allH".equalsIgnoreCase(c)||"bestH".equalsIgnoreCase(c),F="polyhedra".equalsIgnoreCase(c);F&&(c=a.length>(e?5:4)?JS.SV.sValue(a[4]):null);var D="all".equalsIgnoreCase(c)||"allH".equalsIgnoreCase(c),G="best".equalsIgnoreCase(c)||"bestH".equalsIgnoreCase(c);"stddev".equals(c)&&
(c=null);l=c;if(null==c||z||D||G){if(!x&&!h||z&&F)return!1;l="/noaromatic"+(D||G?"/":" nostereo/")+this.e.getSmilesExt().getSmilesMatches(z?"H":"",null,k,null,1,!0,!1)}else D=!0;b=this.e.getSmilesExt().getSmilesCorrelation(k,j,l,n,q,p,null,x,null,null,G,(h?1:2)|(!D&&!G?8:0));if(x){var A=n.size();if(0==A)return d.addXStr("");for(var B=K(q.size()/A),s=new JU.Lst,t=j=0;j<B;j++){var E=JU.AU.newInt2(A);s.addLast(E);for(b=0;b<A;b++,t++)E[b]=M(-1,[n.get(b).i,q.get(t).i])}return D?d.addXList(s):0<s.size()?
d.addXAII(s.get(0)):d.addXStr("")}}else{switch(a.length){case 2:break;case 3:if(e)break;default:return!1}n=this.e.getPointVector(a[0],0);q=this.e.getPointVector(a[1],0);null!=n&&null!=q&&(J.api.Interface.getInterface("JU.Eigen",this.vwr,"script"),b=JU.Measure.getTransformMatrix4(n,q,p,null))}return e||Float.isNaN(b)?d.addXFloat(b):d.addXM4(p.round(1E-7))}catch(L){if(H(L,Exception))return this.e.evalError(null==L.getMessage()?L.toString():L.getMessage(),null),!1;throw L;}},"JS.ScriptMathProcessor,~A");
m(c$,"evaluateConnected",function(d,a,b,c){if(5<a.length)return!1;var e=-2147483648,g=2147483647,f=0,h=3.4028235E38,k=65535,j=null,l=null,m=!1,p=!1;switch(b){case 1275203608:l=-2147483648;f=null;if(0<a.length)switch(a[0].tok){case 2:l=a[0].intValue;break;case 4:f=JS.SV.sValue(a[0])}1275203608==c&&(j=JS.SV.getBitSet(d.getX(),!0));j=C(-1,[Integer.$valueOf(l),f,j]);this.vwr.shm.getShapePropertyData(21,"getCenters",j)||(j[1]=null);return d.addXBs(null==j[1]?new JU.BS:j[1]);case 1228931587:j=d.getX();
if(10!=j.tok||1!=a.length||10!=a[0].tok)return!1;j=j.value;l=a[0].value;a=new JU.Lst;f=this.vwr.ms.at;for(c=j.nextSetBit(0);0<=c;c=j.nextSetBit(c+1)){b=0;h=f[c].bonds;for(p=h.length;0<=--p;)l.get(h[p].getOtherAtom(f[c]).i)&&b++;a.addLast(Integer.$valueOf(b))}return d.addXList(a)}for(c=0;c<a.length;c++){var n=a[c];switch(n.tok){case 10:if((p=r(n.value,JM.BondSet))&&null!=j)return!1;if(null==j)j=n.value;else if(null==l)l=n.value;else return!1;break;case 4:b=JS.SV.sValue(n);k=b.equalsIgnoreCase("hbond")?
30720:JS.ScriptParam.getBondOrderFromString(b);if(131071==k)return!1;break;case 3:m=!0;default:b=n.asInt();n=n.asFloat();if(2147483647!=g)return!1;-2147483648==e?(e=Math.max(b,0),f=n):(g=b,h=n)}}-2147483648==e?(e=1,g=100,f=0.1,h=1E8):2147483647==g&&(g=e,h=f,f=0.1);null==j&&(j=this.vwr.getAllAtoms());m&&null==l&&(l=j);return null!=l?(a=new JU.BS,this.vwr.makeConnections(f,h,k,1086324745,j,l,a,p,!1,0),d.addX(JS.SV.newV(10,JM.BondSet.newBS(a,this.vwr.ms.getAtomIndices(this.vwr.ms.getAtoms(1677721602,
a)))))):d.addXBs(this.vwr.ms.getAtomsConnected(e,g,k,j))},"JS.ScriptMathProcessor,~A,~N,~N");m(c$,"evaluateContact",function(d,a){if(1>a.length||3<a.length)return!1;var b=0,c=100;switch(a[0].tok){case 3:case 2:c=JS.SV.fValue(a[b++]);break;case 10:break;default:return!1}if(b==a.length||!r(a[b].value,JU.BS))return!1;var e=JU.BSUtil.copy(a[b++].value),b=b<a.length?JU.BSUtil.copy(a[b].value):null,c=new J.atomdata.RadiusData(null,10<c?c/100:c,10<c?J.atomdata.RadiusData.EnumType.FACTOR:J.atomdata.RadiusData.EnumType.OFFSET,
J.c.VDW.AUTO),b=this.setContactBitSets(e,b,!0,NaN,c,!1);b.or(e);return d.addXBs(b)},"JS.ScriptMathProcessor,~A");m(c$,"evaluateData",function(d,a){var b=0==a.length?"":JS.SV.sValue(a[0]);switch(a.length){case 0:case 1:break;case 2:case 3:if(10==a[0].tok)return d.addXStr(this.vwr.getModelFileData(b,JS.SV.sValue(a[1]),3==a.length&&JS.SV.bValue(a[2])));break;case 4:var b=a[1].asInt(),c=a[2].asInt(),e=a[3].asInt(),b=JU.Parser.parseFloatArrayFromMatchAndField(JS.SV.sValue(a[0]),null,0,0,null,b,c,null,
e);return d.addXStr(JU.Escape.escapeFloatA(b,!1));default:return!1}if(0==b.indexOf("data2d_"))return b=this.vwr.getDataObj(b,null,2),null==b?d.addXStr(""):2==a.length&&2==a[1].tok?(c=a[1].intValue,0>c&&(c+=b.length),0<=c&&c<b.length?d.addXStr(JU.Escape.escapeFloatA(b[c],!1)):d.addXStr("")):d.addXStr(JU.Escape.escapeFloatAA(b,!1));if(0==b.indexOf("property_"))return b=this.vwr.getDataObj(b,null,1),null==b?d.addXStr(""):d.addXStr(JU.Escape.escapeFloatA(b,!1));b=this.vwr.getDataObj(b,null,-1);return d.addXStr(null==
b?"":""+b[1])},"JS.ScriptMathProcessor,~A");m(c$,"evaluateDotDist",function(d,a,b,c){var e=1275069443==b,g,f=null;switch(a.length){case 2:if(2147483647==c){g=a[0];a=a[1];break}f=a[1];case 1:g=d.getX();a=a[0];break;default:return!1}if(1275069442==b){var h=JU.P3.newP(d.ptValue(g,null));h.cross(h,d.ptValue(a,null));return d.addXPt(h)}var k=7==a.tok?null:d.ptValue(a,null),j=d.planeValue(a);b=NaN;try{if(e){var l=-2147483648==c?0:c&480,m=32==l||64==l,p=480==l;switch(g.tok){case 7:case 10:var n=10==g.tok,
q=10==a.tok,y=8==a.tok,h=n?g.value:null,u=q?a.value:null,v=n?null:g.getList(),r=q?null:a.getList(),C=m&&null!=f&&f.asBoolean();switch(a.tok){case 10:case 7:case 8:var t=this.vwr.ms.at;if(C){var s=NaN,x=2147483647;if(n){for(var w=h.nextSetBit(0);0<=w;w=h.nextSetBit(w+1)){var I=y?t[w].distanceSquared(k):this.e.getBitsetProperty(u,r,c,t[w],j,g.value,null,!1,g.index,!1).floatValue();if(!(32==l?I>=s:I<=s))s=I,x=w}return d.addXBs(2147483647==x?new JU.BS:JU.BSUtil.newAndSetBit(x))}for(w=v.size();0<=--w;){var F=
JS.SV.ptValue(v.get(w)),I=y?F.distanceSquared(k):this.e.getBitsetProperty(u,r,c,F,j,g.value,null,!1,2147483647,!1).floatValue();if(!(32==l?I>=s:I<=s))s=I,x=w}return d.addXInt(x)}if(p){if(null==u){for(var D=z(h.cardinality(),0),s=0,w=h.nextSetBit(0);0<=w;w=h.nextSetBit(w+1),s++)D[s]=t[w].distance(k);return d.addXAF(D)}x=z(h.cardinality(),u.cardinality(),0);s=0;for(w=h.nextSetBit(0);0<=w;w=h.nextSetBit(w+1),s++){c=0;for(var G=u.nextSetBit(0);0<=G;G=u.nextSetBit(G+1),c++)x[s][c]=t[w].distance(t[G])}return d.addXAFF(x)}if(m){D=
z(n?h.cardinality():v.size(),0);if(n){w=h.nextSetBit(0);for(s=0;0<=w;w=h.nextSetBit(w+1))D[s++]=this.e.getBitsetProperty(u,r,c,t[w],j,g.value,null,!1,g.index,!1).floatValue();return d.addXAF(D)}for(w=D.length;0<=--w;)D[w]=this.e.getBitsetProperty(u,r,c,JS.SV.ptValue(v.get(w)),j,null,null,!1,2147483647,!1).floatValue();return d.addXAF(D)}return d.addXObj(this.e.getBitsetProperty(h,v,c,k,j,g.value,null,!1,g.index,!1))}}}var A=d.ptValue(g,null),B=d.planeValue(g);e?b=null!=j&&null!=f?JU.Measure.directedDistanceToPlane(A,
j,JS.SV.ptValue(f)):null==B?null==j?k.distance(A):JU.Measure.distanceToPlane(j,A):JU.Measure.distanceToPlane(B,k):null!=B&&null!=j?b=B.x*j.x+B.y*j.y+B.z*j.z+B.w*j.w:(null!=B?A=JU.P3.new3(B.x,B.y,B.z):null!=j&&(k=JU.P3.new3(j.x,j.y,j.z)),b=A.dot(k))}catch(E){if(!H(E,Exception))throw E;}return d.addXFloat(b)},"JS.ScriptMathProcessor,~A,~N,~N");m(c$,"evaluateHelix",function(d,a){if(1>a.length||5<a.length)return!1;var b=2<a.length?3:1,c=b>=a.length?"array":JS.SV.sValue(a[b]),b=JS.T.getTokFromName(c);
if(2<a.length){var e=d.ptValue(a[0],null),g=d.ptValue(a[1],null);if(0==b||9!=a[2].tok||null==e||null==g)return!1;var f=JU.Quat.newP4(a[2].value),f=JU.Measure.computeHelicalAxis(e,g,f);return null==f?!1:d.addXObj(JU.Escape.escapeHelical(c,b,e,g,f))}c=r(a[0].value,JU.BS)?a[0].value:this.vwr.ms.getAtoms(1094715412,new Integer(a[0].asInt()));switch(b){case 134217751:case 1073741854:case 1665140738:return d.addXObj(this.getHelixData(c,b));case 134217729:return d.addXFloat(this.getHelixData(c,134217729).floatValue());
case 135176:case 1745489939:return d.addXObj(this.getHelixData(c,b));case 1275068418:f=this.getHelixData(c,1073742001);if(null==f)break;return d.addXAS(f)}return!1},"JS.ScriptMathProcessor,~A");m(c$,"getHelixData",function(d,a){var b=d.nextSetBit(0);return 0>b?"null":this.vwr.ms.at[b].group.getHelixData(a,this.vwr.getQuaternionFrame(),this.vwr.getInt(553648144))},"JU.BS,~N");m(c$,"evaluateFind",function(d,a){var b=d.getX(),c=7==b.tok,e=0==a.length,g=e?"":JS.SV.sValue(a[0]),f=1<a.length&&1073742335!=
a[1].tok&&1073742334!=a[1].tok&&10!=a[1].tok?JS.SV.sValue(a[1]):"",h=!c&&g.equalsIgnoreCase("SEQUENCE"),k=!c&&g.equalsIgnoreCase("SEQ");if(g.toUpperCase().startsWith("SMILES/")){g.endsWith("/")||(g+="/");var j=g.substring(6)+"//";JV.JC.isSmilesCanonical(j)?(f="SMILES",g="CHEMICAL"):(g="SMILES",f=j+f)}else g.toUpperCase().startsWith("SMARTS/")&&(g.endsWith("/")||(g+="/"),f=g.substring(6)+(0==f.length?"//":f),g="SMARTS");var l=!c&&g.equalsIgnoreCase("SMILES"),j=!c&&g.equalsIgnoreCase("SMARTS"),m=!c&&
g.equalsIgnoreCase("CHEMICAL"),p=!c&&g.equalsIgnoreCase("MF"),n=!c&&g.equalsIgnoreCase("CELLFORMULA"),q=0<a.length?a[a.length-1]:JS.SV.vF,y=!c&&1073742335==q.tok;try{if(m){var u=10==b.tok?b.value:null,v=null==u?JS.SV.sValue(b):this.vwr.getOpenSmiles(u),v=(0==v.length?"":this.vwr.getChemicalInfo(v,f.toLowerCase(),u)).trim();v.startsWith("InChI")&&(v=JU.PT.rep(JU.PT.rep(v,"InChI=",""),"InChIKey=",""));return d.addXStr(v)}if(l||j||10==b.tok){var u=l||j?2:1,r=u<a.length&&10==a[u].tok?a[u++].value:null,
z="bonds".equalsIgnoreCase(JS.SV.sValue(a[a.length-1])),v=z||y,u=null;switch(b.tok){case 4:var t=JS.SV.sValue(b);if(null!=r||l&&1==a.length)return!1;if(f.equalsIgnoreCase("mf"))u=this.vwr.getSmilesMatcher().getMolecularFormula(t,j);else{var r=f,s=!0,x=!1;switch(a.length){case 4:s=JS.SV.bValue(a[3]);case 3:x=JS.SV.bValue(a[2])}var w=!x&&(!s||!j&&!r.equals("chirality"));try{u=this.e.getSmilesExt().getSmilesMatches(r,t,null,null,j?2:1,!x,!s)}catch(I){if(H(I,Exception))return System.out.println(I.getMessage()),
d.addXInt(-1);throw I;}if(w){var F=u.length;return d.addXInt(!s&&0<F?1:F)}}break;case 10:s=b.value;if(p&&0!=f.length)return d.addXBs(JU.JmolMolecule.getBitSetForMF(this.vwr.ms.at,s,f));if(p||n)return d.addXStr(JU.JmolMolecule.getMolecularFormulaAtoms(this.vwr.ms.at,s,p?null:this.vwr.ms.getCellWeights(s),y));if(h||k){var D=q.asString().equalsIgnoreCase("H"),v=(new Boolean(v|D)).valueOf();return d.addXStr(this.vwr.getSmilesOpt(s,-1,-1,(v?7340032|(D?9437184:0):0)|(k?34603008:1048576),null))}if(l||j)g=
1<a.length&&10==a[1].tok?this.vwr.getSmilesOpt(a[1].value,0,0,0,f):f;f=f.toUpperCase();if(z)var G=this.vwr.getSmilesMatcher().getCorrelationMaps(g,this.vwr.ms.at,this.vwr.ms.ac,s,(l?1:2)|8),u=0<G.length?this.vwr.ms.getDihedralMap(G[0]):M(0,0);else if(f.equalsIgnoreCase("map"))u=G=this.vwr.getSmilesMatcher().getCorrelationMaps(g,this.vwr.ms.at,this.vwr.ms.ac,s,(l?1:2)|128);else if(g.equalsIgnoreCase("crystalClass"))u=this.vwr.ms.generateCrystalClass(s.nextSetBit(0),2!=a.length?null:10==q.tok?this.vwr.ms.getAtomSetCenter(q.value):
JS.SV.ptValue(q));else if(x=(l?0<=f.indexOf("OPEN")?5:1:2)|(y&&0==g.length?22020096:0),0<=f.indexOf("/MOLECULE/")){for(var A=this.vwr.ms.getMolecules(),B=new JU.Lst,t=0;t<A.length;t++)if(A[t].atomList.intersects(s)){var E=this.e.getSmilesExt().getSmilesMatches(g,null,A[t].atomList,r,x,!y,!1);E.isEmpty()||B.addLast(E)}u=B}else u=this.e.getSmilesExt().getSmilesMatches(g,null,s,r,x,!y,!1)}null==u&&this.e.invArg();return d.addXObj(u)}}catch(C){if(H(C,Exception))this.e.evalError(C.getMessage(),null);else throw C;
}w=0<=f.indexOf("v");s=0<=f.indexOf("i");f=0<=f.indexOf("m");F=0==g.length;j=!F&&2==a.length;if(c||j){t=j?this.getPatternMatcher():null;r=null;h=c?b.getList():null;if(j)try{r=t.compile(g,s)}catch(K){if(H(K,Exception))this.e.evalError(K.toString(),null);else throw K;}b=F?null:JS.SV.strListValue(b);B=F?h.size():b.length;JU.Logger.debugging&&JU.Logger.debug("finding "+g);s=new JU.BS;x=0;E=null;A=f?new JU.Lst:null;k="";for(t=0;t<B;t++){if(F)switch(y=h.get(t),y.tok){case 6:y=y.getMap().isEmpty()!=e;break;
case 7:y=0==y.getList().size()!=e;break;case 4:y=0==y.asString().length!=e;break;default:y=!0}else j?(k=b[t],E=r.matcher(k),y=E.find()):y=0<=JS.SV.sValue(h.get(t)).indexOf(g);if(f&&y||!f&&y==!w)x++,s.set(t),f&&A.addLast(w?k.substring(0,E.start())+k.substring(E.end()):E.group())}if(!c)return f?d.addXStr(1==A.size()?A.get(0):""):w?d.addXBool(1==x):f?d.addXStr(0==x?"":E.group()):d.addXInt(0==x?0:E.start()+1);if(f){c=Array(x);if(0<x)for(t=b.length;0<=--t;)s.get(t)&&(--x,c[x]=f?A.get(x):b[t]);return d.addXAS(c)}c=
new JU.Lst;for(t=s.nextSetBit(0);0<=t;t=s.nextSetBit(t+1))c.addLast(h.get(t));return d.addXList(c)}return h?d.addXStr(this.vwr.getJBR().toStdAmino3(JS.SV.sValue(b))):d.addXInt(JS.SV.sValue(b).indexOf(g)+1)},"JS.ScriptMathProcessor,~A");m(c$,"evaluateGetProperty",function(d,a,b,c){var e=a.length,g=c&&1275082245==b,f=c&&1275068437==b&&0<e,h=1275068449==b;b=0;var k=0==e?0:a[0].tok;if(2==e&&(7==k||6==k||14==k))return d.addXObj(this.vwr.extractProperty(a[0].value,a[1].value.toString(),-1));var j=c&&1==
e&&10==a[0].tok?a[0].value:null,l=null==j&&0<e?JS.SV.sValue(a[b++]):"",k=l,m=k.toLowerCase();!g&&0>m.indexOf("[select ")&&(k=m);var p=!1;k.equals("json")&&e>b&&(p=!0,k=JS.SV.sValue(a[b++]));var n=null;if(c){n=d.getX();switch(n.tok){case 10:break;case 4:return b=n.value,e=Array(3),b.startsWith("$")?(b=b.substring(1),a=this.vwr.shm.getShapeIdFromObjectName(b),0<=a&&(e[0]=b,this.vwr.shm.getShapePropertyData(a,"index",e),null!=e[1]&&!l.equals("index")&&(b=e[1].intValue(),e[1]=this.vwr.shm.getShapePropertyIndex(a,
l.intern(),b)))):(a=JV.JC.shapeTokenIndex(JS.T.getTokFromName(b)),0<=a&&(e[0]=l,e[1]=Integer.$valueOf(-1),this.vwr.shm.getShapePropertyData(a,l.intern(),e))),null==e[1]?d.addXStr(""):d.addXObj(e[1]);case 7:if(f){c=n.getList();h=new java.util.Hashtable;k=1<e?JS.SV.sValue(a[e-1]):null;null!=k&&e--;p=Array(e);for(b=0;b<e;b++)p[b]=JS.SV.sValue(a[b]);b=0;for(a=c.size();b<a;b++)if(l=c.get(b),6==l.tok){n=l.getMap();j="";for(g=0;g<e;g++)f=n.get(p[g]),j+=(0==g?"":k)+JS.SV.sValue(f);n=h.get(j);null==n&&h.put(j,
n=JS.SV.newV(7,new JU.Lst));n.getList().addLast(l)}return d.addXMap(h)}if(null!=j){e=n.getList();a=new JU.Lst;for(b=j.nextSetBit(0);0<=b;b=j.nextSetBit(b+1))a.addLast(e.get(b));return d.addXList(a)}default:return g&&(k="[SELECT "+k+"]"),d.addXObj(this.vwr.extractProperty(n,k,-1))}!m.startsWith("bondinfo")&&(!m.startsWith("atominfo")&&!m.startsWith("modelkitinfo"))&&(k="atomInfo."+k)}l="";if(k.equalsIgnoreCase("fileContents")&&2<=e){l=JS.SV.sValue(a[1]);for(b=2;b<e;b++)l+="|"+JS.SV.sValue(a[b]);b=
e}else if(e>b)switch(a[b].tok){case 10:l=a[b++].value;k.equalsIgnoreCase("bondInfo")&&(e>b&&10==a[b].tok)&&(l=C(-1,[l,a[b].value]));break;case 6:case 4:this.vwr.checkPropertyParameter(k)&&(l=a[b++].value)}if(c&&(l=n.value,0>l.nextSetBit(0)))return d.addXStr("");h&&!c&&(k="auxiliaryInfo.models."+k);k=JU.PT.rep(k,".[","[");c=this.vwr.getProperty(null,k,l);b<e&&(c=this.vwr.extractProperty(c,a,b));return d.addXObj(p?JS.SV.safeJSON("value",c):JS.SV.isVariableType(c)?c:JU.Escape.toReadable(k,c))},"JS.ScriptMathProcessor,~A,~N,~B");
m(c$,"evaluateFormat",function(d,a,b,c){var e=2>b.length||1287653388==a?d.getX():null,g=0==b.length?"%U":7==b[0].tok?null:JS.SV.sValue(b[0]);if(!c&&0<b.length&&null!=e&&10!=e.tok&&null!=g){if(2==b.length){var e=e.getList(),f=b[1].getList();if(null==e||null==f)return!1;e=JS.SV.getVariableList(this.getSublist(e,f))}b=C(-1,[b[0],e]);e=null}if(null==e){a=c?-1:JS.SV.getFormatType(g);if(0<=a&&2!=b.length)return!1;if(0<=a||2>b.length||7!=b[1].tok)return b=JS.SV.format(b,a),g.equalsIgnoreCase("json")?d.addXStr(b):
d.addXObj(b);g=b[1].getList();b=C(-1,[b[0],null]);c=Array(g.size());for(e=c.length;0<=--e;)b[1]=g.get(e),c[e]=JS.SV.format(b,a).toString();return d.addXAS(c)}if(7==e.tok&&null==g)return e=e.getList(),f=b[0].getList(),g=this.getSublist(e,f),d.addXList(g);b=10==e.tok?e.value:null;a=JS.T.tokAttr(a,480);return d.addXObj(null==g?"":null==b?JS.SV.sprintf(JU.PT.formatCheck(g),e):this.e.getCmdExt().getBitsetIdent(b,g,e.value,!0,e.index,a))},"JS.ScriptMathProcessor,~N,~A,~B");m(c$,"getSublist",function(d,
a){for(var b=new JU.Lst,c,e,g,f=0,h=d.size();f<h;f++)switch(e=d.get(f),e.tok){case 6:c=e.getMap();g=new JU.Lst;for(var k=0,j=a.size();k<j;k++)e=c.get(JS.SV.sValue(a.get(k))),g.addLast(null==e?JS.SV.newS(""):e);b.addLast(JS.SV.getVariableList(g));break;case 7:c=new java.util.Hashtable;g=e.getList();k=0;for(j=Math.min(g.size(),a.size());k<j;k++)c.put(JS.SV.sValue(a.get(k)),g.get(k));b.addLast(JS.SV.getVariable(c))}return b},"JU.Lst,JU.Lst");m(c$,"evaluateList",function(d,a,b){var c=b.length,e=d.getX(),
g=7==e.tok,f;switch(a){case 1275335685:return 2==c&&d.addX(e.pushPop(b[0],b[1]))||1==c&&d.addX(e.pushPop(null,b[0]));case 1275334681:return 1==c&&d.addX(e.pushPop(b[0],null))||0==c&&d.addX(e.pushPop(null,null));case 1275069441:if(1!=c&&2!=c)return!1;break;case 1275069447:case 1275069446:break;default:if(1!=c)return!1}var h=null,k=null,j=null;if(2==c){var l=JS.SV.sValue(b[0]);f=b[1];if(1275069441==a){h=g?JS.SV.strListValue(e):JU.PT.split(JS.SV.sValue(e),"\n");k=7==f.tok?JS.SV.strListValue(f):JU.PT.split(JS.SV.sValue(f),
"\n");j=Array(c=Math.max(h.length,k.length));for(e=0;e<c;e++)j[e]=(e>=h.length?"":h[e])+l+(e>=k.length?"":k[e]);return d.addXAS(j)}if(1073742335!=f.tok)return!1;j=e.getList();(c=0==l.length)&&(l=",");if(1275069446==a){k=Array(j.size());for(e=j.size();0<=--e;)if(b=j.get(e).getList(),null==b)k[e]=j.get(e);else{a=new JU.SB;g=0;for(h=b.size();g<h;g++)0<g&&a.append(l),f=b.get(g),a.append(c&&4==f.tok?'"'+JU.PT.rep(f.value,'"','""')+'"':""+f.asString());k[e]=JS.SV.newS(a.toString())}return d.addXAV(k)}a=
new JU.Lst;c&&(l="\x00");k=M(2,0);e=0;for(f=j.size();e<f;e++){g=j.get(e).asString();if(c){k[1]=0;for(h=k[0]=0;;){b=JU.PT.getCSVString(g,k);if(null==b){if(-1==k[1]){g+=++e<f?"\n"+j.get(e).asString():'"';k[1]=h;continue}g=g.substring(0,h)+g.substring(h).$replace(",","\x00");break}g=g.substring(0,h)+g.substring(h,k[0]).$replace(",","\x00")+b+g.substring(k[1]);k[1]=h=k[0]+b.length}}for(var m=g.$plit(l),p=new JU.Lst,g=0,h=m.length;g<h;g++){b=m[g];if(0>b.indexOf("."))try{p.addLast(JS.SV.newI(Integer.parseInt(b)));
continue}catch(n){if(!H(n,Exception))throw n;}else try{p.addLast(JS.SV.getVariable(Float.$valueOf(Float.parseFloat(b))));continue}catch(q){if(!H(q,Exception))throw q;}p.addLast(JS.SV.newS(b))}a.addLast(JS.SV.getVariableList(p))}return d.addXObj(JS.SV.getVariableList(a))}f=0==c?JS.SV.newV(1073742327,"all"):b[0];j=1073742327==f.tok;if(!g&&4!=e.tok)return d.binaryOp(this.opTokenFor(a),e,f);var m=JS.SV.isScalar(e),r=JS.SV.isScalar(f),u=p=null,l=e.getList(),v=f.getList();g?c=l.size():m?c=2147483647:(h=
JU.PT.split(JS.SV.sValue(e),"\n"),p=z(c=h.length,0),JU.PT.parseFloatArrayData(h,p));if(j&&1275069446!=a){j=0;if(g)for(e=c;0<=--e;)j+=JS.SV.fValue(l.get(e));else if(!m)for(e=c;0<=--e;)j+=p[e];return d.addXFloat(j)}if(1275069446==a&&4==f.tok){a=new JU.SB;if(m)a.append(JS.SV.sValue(e));else{b=j?"":f.value.toString();for(e=0;e<c;e++)a.append(0<e?b:"").append(JS.SV.sValue(l.get(e)))}return d.addXStr(a.toString())}var C=null;r?C=f:7==f.tok?c=Math.min(c,v.size()):(k=JU.PT.split(JS.SV.sValue(f),"\n"),u=z(k.length,
0),JU.PT.parseFloatArrayData(k,u),c=Math.min(c,u.length));var K=this.opTokenFor(a),t=Array(c);if(g&&j)return j=new JU.Lst,d.addXList(this.addAllLists(e.getList(),j));b=m?e:null;for(var s,e=0;e<c;e++){s=r?C:7==f.tok?v.get(e):Float.isNaN(u[e])?JS.SV.getVariable(JS.SV.unescapePointOrBitsetAsVariable(k[e])):JS.SV.newF(u[e]);m||(b=g?l.get(e):Float.isNaN(p[e])?JS.SV.getVariable(JS.SV.unescapePointOrBitsetAsVariable(h[e])):JS.SV.newF(p[e]));1275069446==a&&7!=b.tok&&(j=new JU.Lst,j.addLast(b),b=JS.SV.getVariableList(j));
if(!d.binaryOp(K,b,s))return!1;t[e]=d.getX()}return d.addXAV(t)},"JS.ScriptMathProcessor,~N,~A");m(c$,"addAllLists",function(d,a){for(var b=d.size(),c=0;c<b;c++){var e=d.get(c);7==e.tok?this.addAllLists(e.getList(),a):a.addLast(e)}return a},"JU.Lst,JU.Lst");m(c$,"evaluateLoad",function(d,a,b){if(1>a.length||3<a.length)return!1;var c=JV.FileManager.fixDOSName(JS.SV.sValue(a[0])),e=1<a.length&&1073742335==a[1].tok,g=this.vwr.async||2<a.length&&1073742335==a[a.length-1].tok,f=1<a.length&&2==a[1].tok?
a[1].asInt():-1;a=1<a.length&&a[1].asString().equalsIgnoreCase("JSON");if(e)return d.addXMap(this.vwr.fm.getFileAsMap(c,null));e=c.startsWith("?");if(JV.Viewer.isJS&&(e||g)){if(b&&e)return d.addXStr("");c=this.e.loadFileAsync("load()_",c,d.oPt,!0)}b=b?this.vwr.fm.getFilePath(c,!1,!1):this.vwr.getFileAsString4(c,f,!1,!1,!0,"script");try{return a?d.addXObj(this.vwr.parseJSON(b)):d.addXStr(b)}catch(h){if(H(h,Exception))return!1;throw h;}},"JS.ScriptMathProcessor,~A,~B");m(c$,"evaluateMath",function(d,
a,b){if(134217749==b)return 1==a.length&&4==a[0].tok?d.addXStr(new java.util.Date+"\t"+JS.SV.sValue(a[0])):d.addXInt((System.currentTimeMillis()&2147483647)-(0==a.length?0:a[0].asInt()));if(1!=a.length)return!1;if(134218250==b)return 2==a[0].tok?d.addXInt(Math.abs(a[0].asInt())):d.addXFloat(Math.abs(a[0].asFloat()));a=JS.SV.fValue(a[0]);switch(b){case 134218242:return d.addXFloat(180*Math.acos(a)/3.141592653589793);case 134218245:return d.addXFloat(Math.cos(3.141592653589793*a/180));case 134218244:return d.addXFloat(Math.sin(3.141592653589793*
a/180));case 134218246:return d.addXFloat(Math.sqrt(a))}return!1},"JS.ScriptMathProcessor,~A,~N");m(c$,"evaluateMeasure",function(d,a,b){var c=0;switch(b){case 1745489939:b=null;for(var e=new JU.Lst,g=z(-1,[3.4028235E38,3.4028235E38]),f=null,h=null,k=!1,j=!1,l=0,m=!1,p=null,n=0,q=3.4028235E38,r=!1,u=!1,p=0;p<a.length;p++)switch(a[p].tok){case 10:var v=a[p].value;0==v.length()&&(m=!0);e.addLast(v);c++;n++;break;case 8:v=new JU.Point3fi;v.setT(a[p].value);e.addLast(v);c++;break;case 2:case 3:g[l++%
2]=JS.SV.fValue(a[p]);break;case 4:v=JS.SV.sValue(a[p]);if(v.startsWith("property_")){b=v;break}v.equalsIgnoreCase("vdw")||v.equalsIgnoreCase("vanderwaals")?q=(p+1<a.length&&2==a[p+1].tok?a[++p].asInt():100)/100:v.equalsIgnoreCase("notConnected")?j=!0:v.equalsIgnoreCase("connected")?k=!0:v.equalsIgnoreCase("minArray")?r=1<=n:v.equalsIgnoreCase("asArray")||0==v.length?u=1<=n:JM.Measurement.isUnits(v)?h=v.toLowerCase():f=c+":"+v;break;default:return!1}if(2>c||4<c||2<l||j&&k)return!1;if(m||3.4028235E38!=
q&&(2!=n||2!=c))return d.addXStr("");p=3.4028235E38==q?new J.atomdata.RadiusData(g,0,null,null):new J.atomdata.RadiusData(null,q,J.atomdata.RadiusData.EnumType.FACTOR,J.c.VDW.AUTO);a=this.vwr.newMeasurementData(null,e).set(0,null,p,b,f,h,null,k,j,null,!0,0,0,null,NaN).getMeasurements(u,r);return d.addXObj(a);case 134217729:if(3!=(c=a.length)&&4!=c)return!1;break;default:if(2!=(c=a.length))return!1}b=Array(c);for(p=0;p<c;p++)if(null==(b[p]=d.ptValue(a[p],null)))return!1;switch(c){case 2:return d.addXFloat(b[0].distance(b[1]));
case 3:return d.addXFloat(JU.Measure.computeAngleABC(b[0],b[1],b[2],!0));case 4:return d.addXFloat(JU.Measure.computeTorsion(b[0],b[1],b[2],b[3],!0))}return!1},"JS.ScriptMathProcessor,~A,~N");m(c$,"evaluateModulation",function(d,a){var b="",c=NaN,e=null;switch(a.length){case 0:break;case 1:switch(a[0].tok){case 8:e=a[0].value;break;case 4:b=a[0].asString();break;default:c=JS.SV.fValue(a[0])}break;case 2:b=JS.SV.sValue(a[0]);c=JS.SV.fValue(a[1]);break;default:return!1}null==e&&1E6>c&&(e=JU.P3.new3(c,
c,c));c=d.getX();c=10==c.tok?c.value:new JU.BS;return d.addXList(this.vwr.ms.getModulationList(c,(b+"D").toUpperCase().charAt(0),e))},"JS.ScriptMathProcessor,~A");m(c$,"evaluatePlane",function(d,a,b){if(134219265==b&&3!=a.length||134217763==b&&2!=a.length&&3!=a.length&&4!=a.length||0==a.length||4<a.length)return!1;var c,e,g,f,h;switch(a.length){case 1:if(10==a[0].tok&&(b=a[0].value,3==b.cardinality()))return a=this.vwr.ms.getAtomPointVector(b),d.addXPt4(JU.Measure.getPlaneThroughPoints(a.get(0),a.get(1),
a.get(2),new JU.V3,new JU.V3,new JU.P4));a=JU.Escape.uP(JS.SV.sValue(a[0]));return r(a,JU.P4)?d.addXPt4(a):d.addXStr(""+a);case 2:if(134217763==b){if(9!=a[1].tok)return!1;b=new JU.P3;f=new JU.V3;h=new JU.V3;g=a[1].value;if(9==a[0].tok)return a=JU.Measure.getIntersectionPP(a[0].value,g),null==a?d.addXStr(""):d.addXList(a);e=d.ptValue(a[0],null);return null==e?d.addXStr(""):d.addXPt(JU.Measure.getIntersection(e,null,g,b,f,h))}case 3:case 4:switch(b){case 134219265:return d.addXPt4(this.e.getHklPlane(JU.P3.new3(JS.SV.fValue(a[0]),
JS.SV.fValue(a[1]),JS.SV.fValue(a[2]))));case 134217763:c=d.ptValue(a[0],null);e=d.ptValue(a[1],null);if(null==c||null==e)return d.addXStr("");e=JU.V3.newV(e);e.normalize();if(9==a[2].tok)return b=new JU.P3,f=new JU.V3,h=new JU.V3,c=JU.Measure.getIntersection(c,e,a[2].value,b,f,h),null==c?d.addXStr(""):d.addXPt(c);b=d.ptValue(a[2],null);if(null==b)return d.addXStr("");f=new JU.V3;b=JU.P3.newP(b);if(3==a.length)return JU.Measure.projectOntoAxis(b,c,e,f),d.addXPt(b);h=JS.SV.fValue(a[3]);a=JU.P3.newP(b);
JU.Measure.projectOntoAxis(b,c,e,f);a=a.distance(b);c=new JU.Lst;a==h?c.addLast(b):a<h&&(a=Math.sqrt(h*h-a*a),f.scaleAdd2(a,e,b),c.addLast(JU.P3.newP(f)),f.scaleAdd2(-a,e,b),c.addLast(JU.P3.newP(f)));return d.addXList(c)}switch(a[0].tok){case 2:case 3:if(3==a.length)return h=JS.SV.fValue(a[0]),b=JS.SV.fValue(a[1]),a=JS.SV.fValue(a[2]),f=JU.V3.new3(0,0,1),e=JU.P3.new3(0,1,0),a=JU.Quat.newVA(e,a),a.getMatrix().rotate(f),e.set(0,0,1),a=JU.Quat.newVA(e,b),a.getMatrix().rotate(f),e.setT(f),e.scale(h),
g=new JU.P4,JU.Measure.getPlaneThroughPoint(e,f,g),d.addXPt4(g);break;case 10:case 8:c=d.ptValue(a[0],null);e=d.ptValue(a[1],null);if(null==e)return!1;b=2<a.length&&(10==a[2].tok||8==a[2].tok)?d.ptValue(a[2],null):null;f=JU.V3.newV(e);if(null==b)return g=new JU.P4,2==a.length||2!=a[2].tok&&3!=a[2].tok&&!a[2].asBoolean()?(b=JU.P3.newP(c),b.add(e),b.scale(0.5),f.sub(c),f.normalize()):1073742335==a[2].tok?b=c:(f.sub(c),b=new JU.P3,b.scaleAdd2(a[2].asFloat(),f,c)),JU.Measure.getPlaneThroughPoint(b,f,
g),d.addXPt4(g);h=new JU.V3;a=4==a.length?d.ptValue(a[3],null):null;a=JU.Measure.getDirectedNormalThroughPoints(c,e,b,a,f,h);return d.addXPt4(JU.P4.new4(f.x,f.y,f.z,a))}}if(4!=a.length)return!1;b=JS.SV.fValue(a[0]);f=JS.SV.fValue(a[1]);h=JS.SV.fValue(a[2]);a=JS.SV.fValue(a[3]);return d.addXPt4(JU.P4.new4(b,f,h,a))},"JS.ScriptMathProcessor,~A,~N");m(c$,"evaluatePoint",function(d,a){switch(a.length){default:return!1;case 1:if(3==a[0].tok||2==a[0].tok)return d.addXInt(a[0].asInt());var b=null;if(7==
a[0].tok){var c=a[0].getList(),e=c.size();if(0==e)return!1;switch(c.get(0).tok){case 4:b=c.get(0).value;if(!b.startsWith("{")||r(JU.Escape.uP(b),String))break;for(var b=new JU.Lst,g=0;g<e;g++)b.addLast(JS.SV.getVariable(JU.Escape.uP(JS.SV.sValue(c.get(g)))));return d.addXList(b)}b="{"+JS.SV.sValue(a[0])+"}"}null==b&&(b=JS.SV.sValue(a[0]));c=JU.Escape.uP(b);return r(c,JU.P3)?d.addXPt(c):d.addXStr(""+c);case 2:switch(a[1].tok){case 1073742334:case 1073742335:switch(a[0].tok){case 8:c=JU.P3.newP(a[0].value);
break;case 10:c=this.vwr.ms.getAtomSetCenter(a[0].value);break;default:return!1}1073742335==a[1].tok?(this.vwr.tm.transformPt3f(c,c),c.y=this.vwr.tm.height-c.y,this.vwr.antialiased&&c.scale(0.5)):(this.vwr.antialiased&&c.scale(2),c.y=this.vwr.tm.height-c.y,this.vwr.tm.unTransformPoint(c,c));break;case 8:e=a[0].getList();if(null==e||4!=e.size())return!1;b=JS.SV.ptValue(a[1]);c=JU.P3.newP(JS.SV.ptValue(e.get(0)));c.scaleAdd2(b.x,JS.SV.ptValue(e.get(1)),c);c.scaleAdd2(b.y,JS.SV.ptValue(e.get(2)),c);
c.scaleAdd2(b.z,JS.SV.ptValue(e.get(3)),c);break;default:return!1}return d.addXPt(c);case 3:return d.addXPt(JU.P3.new3(a[0].asFloat(),a[1].asFloat(),a[2].asFloat()));case 4:return d.addXPt4(JU.P4.new4(a[0].asFloat(),a[1].asFloat(),a[2].asFloat(),a[3].asFloat()))}},"JS.ScriptMathProcessor,~A");m(c$,"evaluatePrompt",function(d,a){if(1!=a.length&&2!=a.length&&3!=a.length)return!1;var b=JS.SV.sValue(a[0]),c=1<a.length&&7==a[1].tok?JS.SV.strListValue(a[1]):null,e=null!=c||1==a.length||3==a.length&&a[2].asBoolean(),
g=null!=c?null:2<=a.length?JS.SV.sValue(a[1]):"OK",b=""+this.vwr.prompt(b,g,c,e);return e&&null!=c?d.addXInt(Integer.parseInt(b)+1):d.addXStr(b)},"JS.ScriptMathProcessor,~A");m(c$,"evaluateQuaternion",function(d,a,b){var c=null,e=a.length,g=2147483647,f=!1;134221850==b&&(1<e&&(4==a[e-1].tok&&a[e-1].value.equalsIgnoreCase("relative"))&&(e--,f=!0),1<e&&(2==a[e-1].tok&&10==a[0].tok)&&(g=a[e-1].asInt(),0>=g&&(g=2147483646),e--));switch(e){case 0:case 1:case 4:break;case 2:if(134221850==b){if(7==a[0].tok&&
(7==a[1].tok||1073742335==a[1].tok))break;if(10==a[0].tok&&(2==a[1].tok||10==a[1].tok))break}if(null==(c=d.ptValue(a[0],null))||134221850!=b&&8==a[1].tok)return!1;break;case 3:if(134221850!=b)return!1;if(9==a[0].tok){if(8!=a[2].tok&&10!=a[2].tok)return!1;break}for(var h=0;3>h;h++)if(8!=a[h].tok&&10!=a[h].tok)return!1;break;default:return!1}var k=h=null,j=null;switch(e){case 0:return d.addXPt4(this.vwr.tm.getRotationQ().toPoint4f());default:if(134221850==b&&7==a[0].tok){b=this.e.getQuaternionArray(a[0].getList(),
1073742001);a=JU.Quat.sphereMean(b,null,1E-4);h=r(a,JU.Quat)?a:null;break}else if(134221850==b&&10==a[0].tok)k=this.vwr.getAtomGroupQuaternions(a[0].value,g);else if(11==a[0].tok)h=JU.Quat.newM(a[0].value);else if(9==a[0].tok)j=a[0].value;else if(j=JS.SV.sValue(a[0]),j=JU.Escape.uP(j.equalsIgnoreCase("best")?this.vwr.getOrientationText(1073741864,"best",null).toString():j),!r(j,JU.P4))return!1;134217731==b&&(h=JU.Quat.newVA(JU.P3.new3(j.x,j.y,j.z),j.w));break;case 2:if(134221850==b){if(7==a[0].tok&&
7==a[1].tok){b=this.e.getQuaternionArray(a[0].getList(),1073742001);k=this.e.getQuaternionArray(a[1].getList(),1073742001);k=JU.Quat.div(k,b,g,f);break}if(7==a[0].tok&&1073742335==a[1].tok)return b=this.e.getQuaternionArray(a[0].getList(),1073742001),g=z(1,0),JU.Quat.sphereMean(b,g,1E-4),d.addXFloat(g[0]);if(10==a[0].tok&&10==a[1].tok){b=this.vwr.getAtomGroupQuaternions(a[0].value,2147483647);k=this.vwr.getAtomGroupQuaternions(a[1].value,2147483647);k=JU.Quat.div(k,b,g,f);break}}h=d.ptValue(a[1],
null);j=d.planeValue(a[0]);h=null!=h?JU.Quat.getQuaternionFrame(JU.P3.new3(0,0,0),c,h):JU.Quat.newVA(c,JS.SV.fValue(a[1]));break;case 3:if(9==a[0].tok)return g=8==a[2].tok?a[2].value:this.vwr.ms.getAtomSetCenter(a[2].value),d.addXStr(JU.Escape.drawQuat(JU.Quat.newP4(a[0].value),"q",JS.SV.sValue(a[1]),g,1));b=Array(3);for(h=0;3>h;h++)b[h]=8==a[h].tok?a[h].value:this.vwr.ms.getAtomSetCenter(a[h].value);h=JU.Quat.getQuaternionFrame(b[0],b[1],b[2]);break;case 4:134221850==b?j=JU.P4.new4(JS.SV.fValue(a[1]),
JS.SV.fValue(a[2]),JS.SV.fValue(a[3]),JS.SV.fValue(a[0])):h=JU.Quat.newVA(JU.P3.new3(JS.SV.fValue(a[0]),JS.SV.fValue(a[1]),JS.SV.fValue(a[2])),JS.SV.fValue(a[3]))}if(null!=k){if(2147483647!=g){g=new JU.Lst;for(h=0;h<k.length;h++)g.addLast(k[h].toPoint4f());return d.addXList(g)}h=0<k.length?k[0]:null}return d.addXPt4((null==h?JU.Quat.newP4(j):h).toPoint4f())},"JS.ScriptMathProcessor,~A,~N");m(c$,"evaluateRandom",function(d,a){if(3<a.length)return!1;null==this.rand&&(this.rand=new java.util.Random);
var b=0,c=1;switch(a.length){case 3:this.rand.setSeed(N(JS.SV.fValue(a[2])));case 2:c=JS.SV.fValue(a[1]);case 1:b=JS.SV.fValue(a[0]);case 0:break;default:return!1}return d.addXFloat(this.rand.nextFloat()*(c-b)+b)},"JS.ScriptMathProcessor,~A");m(c$,"evaluateRowCol",function(d,a,b){if(1!=a.length)return!1;a=a[0].asInt()-1;var c=d.getX();switch(c.tok){case 11:if(0>a||2<a)break;c=c.value;switch(b){case 1275068935:return b=z(3,0),c.getRow(a,b),d.addXAF(b);default:return b=z(3,0),c.getColumn(a,b),d.addXAF(b)}case 12:if(0>
a||2<a)break;c=c.value;switch(b){case 1275068935:return b=z(4,0),c.getRow(a,b),d.addXAF(b);default:return b=z(4,0),c.getColumn(a,b),d.addXAF(b)}case 7:b=c.getList();for(var c=new JU.Lst,e=0,g=b.size();e<g;e++){var f=b.get(e).getList();if(null==f)return d.addXStr("");c.addLast(a<f.size()?f.get(a):JS.SV.newS(""))}return d.addXList(c)}return!1},"JS.ScriptMathProcessor,~A,~N");m(c$,"evaluateIn",function(d,a){var b=d.getX();switch(a.length){case 1:var c=a[0].getList();if(null!=c)for(var e=0,g=c.size();e<
g;e++)if(JS.SV.areEqual(b,c.get(e)))return d.addXInt(e+1);break;default:for(e=0;e<a.length;e++)if(JS.SV.areEqual(b,a[e]))return d.addXInt(e+1)}return d.addXInt(0)},"JS.ScriptMathProcessor,~A");m(c$,"evaluateReplace",function(d,a){var b=!1,c,e;switch(a.length){case 0:b=!0;c=e=null;break;case 3:b=JS.SV.bValue(a[2]);case 2:c=JS.SV.sValue(a[0]);e=JS.SV.sValue(a[1]);break;default:return!1}var g=d.getX();if(7==g.tok){for(var g=JS.SV.strListValue(g),f=Array(g.length),h=g.length;0<=--h;)f[h]=null==c?JU.PT.clean(g[h]):
b?JU.PT.replaceAllCharacters(g[h],c,e):JU.PT.rep(g[h],c,e);return d.addXAS(f)}g=JS.SV.sValue(g);return d.addXStr(null==c?JU.PT.clean(g):b?JU.PT.replaceAllCharacters(g,c,e):JU.PT.rep(g,c,e))},"JS.ScriptMathProcessor,~A");m(c$,"evaluateScript",function(d,a,b){if((134222350==b||134238732==b)&&1!=a.length||0==a.length)return!1;var c=JS.SV.sValue(a[0]),e=new JU.SB;switch(b){case 134218759:return 2==a.length?c.equalsIgnoreCase("JSON")&&d.addXObj(this.vwr.parseJSONMap(JS.SV.sValue(a[1]))):d.addXObj(this.vwr.evaluateExpressionAsVariable(c));
case 134222850:a=2==a.length?JS.SV.sValue(a[1]):".";a.equals(".")||e.append(this.vwr.jsEval(a+"\u0001"+c));(a.equals(".")||a.equals("*"))&&this.e.runScriptBuffer(c,e,!0);break;case 134222350:this.e.runScriptBuffer("show "+c,e,!0);break;case 134238732:return d.addX(this.vwr.jsEvalSV(c))}var c=e.toString(),g;return Float.isNaN(g=JU.PT.parseFloatStrict(c))?d.addXStr(c):0<=c.indexOf(".")?d.addXFloat(g):d.addXInt(JU.PT.parseInt(c))},"JS.ScriptMathProcessor,~A,~N");m(c$,"evaluateSort",function(d,a,b){if(1<
a.length)return!1;if(1275068444==b){if(1==a.length&&4==a[0].tok)return d.addX(d.getX().sortMapArray(a[0].asString()));var c=0==a.length?0:a[0].asInt();return d.addX(d.getX().sortOrReverse(c))}b=d.getX();a=0==a.length?null:a[0];if(4==b.tok){var c=0,e=JS.SV.sValue(b);if(null==a)return d.addXInt(0);a=JS.SV.sValue(a);for(b=0;b<e.length;b++){b=e.indexOf(a,b);if(0>b)break;c++}return d.addXInt(c)}var c=new JU.Lst,g=e=null,f=JS.SV.getVariable(b.value).sortOrReverse(0).getList();if(null==f)return null==a?
d.addXStr(""):d.addXInt(0);b=0;for(var h=f.size();b<=h;b++){var k=b==h?null:f.get(b);if(!(null!=a&&null!=k&&!JS.SV.areEqual(k,a)))if(JS.SV.areEqual(k,e))g.intValue++;else{if(null!=e){var j=new JU.Lst;j.addLast(e);j.addLast(g);c.addLast(JS.SV.getVariableList(j))}g=JS.SV.newI(1);e=k}}return null==a?d.addX(JS.SV.getVariableList(c)):c.isEmpty()?d.addXInt(0):d.addX(c.get(0).getList().get(1))},"JS.ScriptMathProcessor,~A,~N");m(c$,"evaluateString",function(d,a,b){var c=d.getX(),e=0<b.length?JS.SV.sValue(b[0]):
1275068932==a?"":"\n";switch(b.length){case 0:break;case 1:if(1073742335==b[0].tok)return d.addX(JS.SV.getVariable(JU.PT.getTokens(c.asString())));break;case 2:if(7==c.tok)break;if(1275069447==a){c=JS.SV.getVariable(JU.PT.split(JU.PT.rep(c.value,"\n\r","\n").$replace("\r","\n"),"\n"));break}default:return!1}if(7==c.tok&&1275068932!=a&&(1275069447!=a||2==b.length))return d.addX(c),this.evaluateList(d,a,b);b=1275069447==a&&10==c.tok||1275068932==a&&7==c.tok?null:JS.SV.sValue(c);switch(a){case 1275069447:if(10==
c.tok){e=c.value;c=this.vwr.ms.mc;b=new JU.Lst;for(a=0;a<c;a++){var g=this.vwr.getModelUndeletedAtomsBitSet(a);g.and(e);b.addLast(JS.SV.getVariable(g))}return d.addXList(b)}return d.addXAS(JU.PT.split(b,e));case 1275069446:return 0<b.length&&"\n"==b.charAt(b.length-1)&&(b=b.substring(0,b.length-1)),d.addXStr(JU.PT.rep(b,"\n",e));case 1275068932:if(null!=b)return d.addXStr(JU.PT.trim(b,e));c=JS.SV.strListValue(c);for(a=c.length;0<=--a;)c[a]=JU.PT.trim(c[a],e);return d.addXAS(c)}return d.addXStr("")},
"JS.ScriptMathProcessor,~N,~A");m(c$,"evaluateSubstructure",function(d,a,b,c){if(0==a.length||c&&1<a.length)return!1;var e=new JU.BS,g=JS.SV.sValue(a[0]);if(0<g.length)try{var f=c?d.getX().value:2==a.length&&10==a[1].tok?a[1].value:null,e=this.vwr.getSmilesMatcher().getSubstructureSet(g,this.vwr.ms.at,this.vwr.ms.ac,f,134218757==b?1:2)}catch(h){if(H(h,Exception))this.e.evalError(h.getMessage(),null);else throw h;}return d.addXBs(e)},"JS.ScriptMathProcessor,~A,~N,~B");m(c$,"evaluateSymop",function(d,
a,b){b=b?d.getX():null;if(null!=b&&10!=b.tok)return!1;b=null==b?null:b.value;null==b&&1==this.vwr.ms.mc&&(b=this.vwr.getModelUndeletedAtomsBitSet(0));var c=a.length;if(0==c){if(b.isEmpty())return!1;a=JU.PT.split(JU.PT.trim(this.vwr.getSymTemp().getSpaceGroupInfo(this.vwr.ms,null,this.vwr.ms.at[b.nextSetBit(0)].mi,!1,null).get("symmetryInfo"),"\n"),"\n");b=new JU.Lst;for(var e=0,g=a.length;e<g;e++)b.addLast(JU.PT.split(a[e],"\t"));return d.addXList(b)}var e=null,g=-2147483648,f=0;switch(a[0].tok){case 4:e=
JS.SV.sValue(a[0]);f++;break;case 12:e=a[0].escape();f++;break;case 2:g=a[0].asInt(),f++}null==b&&(f<c&&10==a[f].tok&&(b=new JU.BS).or(a[f].value),f+1<c&&10==a[f+1].tok&&(null==b?b=new JU.BS:b).or(a[f+1].value));var h=null,k=null;null!=(h=c>f?d.ptValue(a[f],b):null)&&f++;null!=(k=c>f?d.ptValue(a[f],b):null)&&f++;var j=null!=k&&a.length>f&&-2147483648==g&&2==a[f].tok?a[f].intValue:-1;0<=j&&f++;-2147483648==g&&(g=0);c=c==f?null!=k?"all":null!=h?"point":"matrix":JS.SV.sValue(a[f++]).toLowerCase();return null!=
b&&!b.isEmpty()&&f==a.length&&d.addXObj(this.vwr.getSymmetryInfo(b.nextSetBit(0),e,g,h,k,0,c,0,j,0))},"JS.ScriptMathProcessor,~A,~B");m(c$,"evaluateTensor",function(d,a){var b=2==a.length&&1275068445==a[1].tok,c=b?null:d.getX();if(2<a.length||!b&&10!=c.tok)return!1;var c=c.value,e=b||0==a.length?null:JS.SV.sValue(a[0]).toLowerCase(),g=this.vwr.getNMRCalculation();if("unique".equals(e))return d.addXBs(g.getUniqueTensorSet(c));var f=2>a.length?null:JS.SV.sValue(a[1]).toLowerCase();return b?d.addXObj(a[0].value.getInfo(f)):
d.addXList(g.getTensorInfo(e,f,c))},"JS.ScriptMathProcessor,~A");m(c$,"evaluateUserFunction",function(d,a,b,c,e){var g=null;if(e)switch(g=d.getX(),g.tok){case 10:break;case 6:if(0<b.length)return!1;g=g.getMap().get(a);return null==g?d.addXStr(""):d.addX(g);default:return!1}a=a.toLowerCase();d.wasX=!1;for(var f=new JU.Lst,h=0;h<b.length;h++)f.addLast(b[h]);if(e)return d.addXObj(this.e.getBitsetProperty(g.value,null,c,null,null,g.value,C(-1,[a,f]),!1,g.index,!1));a=this.e.getUserFunctionResult(a,f,
null);return null==a?!1:d.addX(a)},"JS.ScriptMathProcessor,~S,~A,~N,~B");m(c$,"evaluateWithin",function(d,a){if(1>a.length||5<a.length)return!1;var b=a.length;if(1==b&&10==a[0].tok)return d.addX(a[0]);var c=0,e=a[0].value,g=""+e,f=a[0].tok;4==f&&(f=JS.T.getTokFromName(g));var h=this.vwr.ms,k=!1,j=!1,l=!1,m=!1,p=null;switch(f){case 1648363544:k=!0,e=null;case 3:case 2:m=!0;if(2>b||3==b&&7==a[1].tok&&7!=a[2].tok)return!1;c=k?100:JS.SV.fValue(a[0]);switch(f=a[1].tok){case 1073742335:case 1073742334:j=
a[1].asBoolean();2<b&&JS.SV.sValue(a[2]).equalsIgnoreCase("unitcell")&&(f=1814695966);b=0;break;case 4:f=JS.SV.sValue(a[1]);if(f.startsWith("$"))return d.addXBs(this.getAtomsNearSurface(c,f.substring(1)));if(f.equalsIgnoreCase("group"))l=!0,f=1086324742;else if(f.equalsIgnoreCase("vanderwaals")||f.equalsIgnoreCase("vdw"))e=null,k=!0,f=1648363544;else if(f.equalsIgnoreCase("unitcell"))f=1814695966;else return!1}break;case 7:1==b&&(e=a[0].asString(),f=0);break;case 1073742328:return 3==b&&r(a[1].value,
JU.BS)&&r(a[2].value,JU.BS)&&d.addXBs(this.vwr.getBranchBitSet(a[2].value.nextSetBit(0),a[1].value.nextSetBit(0),!0));case 134218757:case 1237320707:case 134218756:c=null;e=!0;switch(b){case 2:break;case 3:if(e=10==a[2].tok)c=a[2].value;break;default:e=!1}e||this.e.invArg();return d.addXObj(this.e.getSmilesExt().getSmilesMatches(JS.SV.sValue(a[1]),null,c,null,134218756==f?2:1,d.asBitSet,!1))}if(r(e,String)){if(0==f){f=1073742362;if(2<b)return!1;b=2}}else if(!m)return!1;switch(b){case 1:switch(f){case 136314895:case 2097184:case 1678381065:return d.addXBs(h.getAtoms(f,
null));case 1073741863:return d.addXBs(h.getAtoms(f,""));case 1073742362:return d.addXBs(h.getAtoms(1086324744,g))}return!1;case 2:switch(f){case 1073742362:f=1086324744;break;case 1073741824:case 1086326786:case 1086326785:case 1073741863:case 1086324744:case 1111490587:case 1073742128:case 1073741925:case 1073742189:return d.addXBs(this.vwr.ms.getAtoms(f,JS.SV.sValue(a[a.length-1])))}break;case 3:switch(f){case 1073742335:case 1073742334:case 1086324742:case 1648363544:case 1814695966:case 134217750:case 134219265:case 1073742329:case 8:case 7:break;
case 1086324744:g=JS.SV.sValue(a[2]);break;default:return!1}}var n=null,e=b=null,q=a.length-1;switch(a[q].tok){case 9:n=a[q].value;break;case 8:b=a[q].value;JS.SV.sValue(a[1]).equalsIgnoreCase("hkl")&&(n=this.e.getHklPlane(b));break;case 7:e=2==q&&7==a[1].tok?a[1].getList():null,b=2==q?JS.SV.ptValue(a[1]):1==q?JU.P3.new3(NaN,0,0):null}if(null!=n)return d.addXBs(h.getAtomsNearPlane(c,n));h=10==a[q].tok?a[q].value:null;if(0<q&&null==b&&null==e&&null==h)return!1;if(1814695966==f)return(null!=h||null!=
b)&&d.addXObj(this.vwr.ms.getUnitCellPointsWithin(c,h,b,j));if(null!=b||null!=e){if(7==a[q].tok){k=a[q].getList();f=Array(k.size());for(g=f.length;0<=--g;)f[g]=JS.SV.ptValue(k.get(g));k=null;if(null!=e){k=Array(e.size());for(g=k.length;0<=--g;)k[g]=JS.SV.ptValue(e.get(g))}e=Array(1);switch(J.bspt.PointIterator.withinDistPoints(c,b,f,k,e)){case 134217751:return d.addXPt(e[0]);case 1073742001:return d.addXList(e[0]);case 1275068418:return d.addXAI(e[0]);case 4:return d.addXStr(e[0]);default:return!1}}return d.addXBs(this.vwr.getAtomsNearPt(c,
b))}if(1086324744==f)return d.addXBs(this.vwr.ms.getSequenceBits(g,h,new JU.BS));null==h&&(h=new JU.BS);if(!m)return d.addXBs(this.vwr.ms.getAtoms(f,h));if(l)return d.addXBs(this.vwr.getGroupsWithin(N(c),h));k&&(p=new J.atomdata.RadiusData(null,10<c?c/100:c,10<c?J.atomdata.RadiusData.EnumType.FACTOR:J.atomdata.RadiusData.EnumType.OFFSET,J.c.VDW.AUTO),0>c&&(c=0));return d.addXBs(this.vwr.ms.getAtomsWithinRadius(c,h,j,p))},"JS.ScriptMathProcessor,~A");m(c$,"evaluateWrite",function(d,a){switch(a.length){case 0:return!1;
case 1:var b=a[0].asString().toUpperCase();if(b.equals("PNGJ"))return d.addXMap(this.vwr.fm.getFileAsMap(null,"PNGJ"));if(JU.PT.isOneOf(b,";ZIP;ZIPALL;JMOL;")){var c=new java.util.Hashtable,e=new JU.OC;c.put("outputChannel",e);this.vwr.createZip(null,b,c);b=JU.Rdr.getBIS(e.toByteArray());c=new java.util.Hashtable;this.vwr.getJzt().readFileAsMap(b,c,null);return d.addXMap(c)}}return d.addXStr(this.e.getCmdExt().dispatch(134221856,!0,a))},"JS.ScriptMathProcessor,~A");m(c$,"getAtomsNearSurface",function(d,
a){var b=C(-1,[a,null,null]);if(this.e.getShapePropertyData(24,"getVertices",b))return this.getAtomsNearPts(d,b[1],b[2]);b[1]=Integer.$valueOf(0);b[2]=Integer.$valueOf(-1);if(this.e.getShapePropertyData(22,"getCenter",b))return this.vwr.getAtomsNearPt(d,b[2]);b[1]=Float.$valueOf(d);return this.e.getShapePropertyData(21,"getAtomsWithin",b)?b[2]:new JU.BS},"~N,~S");m(c$,"getAtomsNearPts",function(d,a,b){var c=new JU.BS;if(0==a.length||null!=b&&b.isEmpty())return c;null==b&&(b=JU.BSUtil.setAll(a.length));
for(var e=this.vwr.ms.at,g=this.vwr.ms.ac;0<=--g;)for(var f=e[g],h=b.nextSetBit(0);0<=h;h=b.nextSetBit(h+1))if(f.distance(a[h])<d){c.set(g);break}return c},"~N,~A,JU.BS");m(c$,"getMinMax",function(d,a){for(var b=null,c=null,e=0,g=null;;){if(JU.AU.isAF(d)){if(1275068437==a)break;b=d;e=b.length;if(0==e)break}else if(r(d,JU.Lst))if(c=d,e=c.size(),0==e){if(1275068437!=a)break}else{if(1275068437!=a){var f=c.get(0);if(8==f.tok)return this.getMinMaxPoint(c,a);if(4==f.tok&&f.value.startsWith("{")){b=JS.SV.ptValue(f);
if(r(b,JU.P3))return this.getMinMaxPoint(c,a);if(r(b,JU.P4))return this.getMinMaxQuaternion(c,a);break}}}else break;var h,k=!1;switch(a){case 1275068437:g=new java.util.Hashtable;f=h=0;break;case 32:k=!0;f=3.4028235E38;h=2147483647;break;case 64:f=-3.4028235E38;h=-2147483647;break;default:f=h=0}for(var j=0,l=0,m=!0,p=1275068437==a;0<=--e;){var n=null==c?JS.SV.vF:c.get(e),q=p?1:null==b?JS.SV.fValue(n):b[e];if(!Float.isNaN(q))switch(l++,a){case 160:case 192:j+=q*q;case 128:case 96:f+=q;break;case 1275068437:m=
(new Boolean(m&2==n.tok)).valueOf();n=n.asString();q=g.get(n);g.put(n,null==q?new Integer(1):new Integer(q.intValue()+1));break;case 32:case 64:m=(new Boolean(m&2==n.tok)).valueOf(),k==q<f&&(f=q,m&&(h=n.intValue))}}if(1275068437==a)return g;if(0==l)break;switch(a){case 96:f/=l;break;case 192:if(1==l)break;f=Math.sqrt((j-f*f/l)/(l-1));break;case 32:case 64:if(m)return Integer.$valueOf(h);break;case 160:f=j}return Float.$valueOf(f)}return"NaN"},"~O,~N");m(c$,"getMinMaxPoint",function(d,a){var b=null,
c=null,e=0;r(d,Array)?(b=d,e=b.length):r(d,JU.Lst)&&(c=d,e=c.size());if(null==c&&null==b)return"NaN";for(var g=new JU.P3,f=z(e,0),h=0;3>h;h++){for(var k=0;k<e;k++){var j=null==b?JS.SV.ptValue(c.get(k)):b[k];if(null==j)return"NaN";switch(h){case 0:f[k]=j.x;break;case 1:f[k]=j.y;break;case 2:f[k]=j.z}}k=this.getMinMax(f,a);if(!r(k,Number))return"NaN";k=k.floatValue();switch(h){case 0:g.x=k;break;case 1:g.y=k;break;case 2:g.z=k}}return g},"~O,~N");m(c$,"getMinMaxQuaternion",function(d,a){var b;switch(a){case 32:case 64:case 128:case 160:return"NaN"}for(;;){b=
this.e.getQuaternionArray(d,1073742001);if(null==b)break;var c=z(1,0);b=JU.Quat.sphereMean(b,c,1E-4);switch(a){case 96:return b;case 192:return Float.$valueOf(c[0])}break}return"NaN"},"JU.Lst,~N");m(c$,"getPatternMatcher",function(){return null==this.pm?this.pm=J.api.Interface.getUtil("PatternMatcher",this.e.vwr,"script"):this.pm});m(c$,"opTokenFor",function(d){switch(d){case 1275069441:case 1275069446:return JS.T.tokenPlus;case 1275068931:return JS.T.tokenMinus;case 1275068929:return JS.T.tokenTimes;
case 1275068930:return JS.T.tokenMul3;case 1275068928:return JS.T.tokenDivide}return null},"~N");m(c$,"setContactBitSets",function(d,a,b,c,e,g){var f,h;null==a?(a=JU.BSUtil.setAll(this.vwr.ms.ac),JU.BSUtil.andNot(a,this.vwr.slm.bsDeleted),a.andNot(d),f=!1):(h=JU.BSUtil.copy(d),h.or(a),h=this.vwr.ms.getModelBS(h,!1).cardinality(),f=1<h,g&&(1<h&&!this.e.tQuiet)&&this.e.showString(J.i18n.GT.$("Note: More than one model is involved in this contact!")));if(!d.equals(a)){if(g=!b||d.cardinality()<a.cardinality())h=
this.vwr.ms.getAtomsWithinRadius(c,d,f,Float.isNaN(c)?e:null),a.and(h);b&&(h=this.vwr.ms.getAtomsWithinRadius(c,a,f,Float.isNaN(c)?e:null),d.and(h),g||(h=this.vwr.ms.getAtomsWithinRadius(c,d,f,Float.isNaN(c)?e:null),a.and(h)),h=JU.BSUtil.copy(a),h.and(d),h.equals(d)?a.andNot(d):h.equals(a)&&d.andNot(a))}return a},"JU.BS,JU.BS,~B,~N,J.atomdata.RadiusData,~B")})})(Clazz,Clazz.getClassName,Clazz.newLongArray,Clazz.doubleToByte,Clazz.doubleToInt,Clazz.doubleToLong,Clazz.declarePackage,Clazz.instanceOf,
Clazz.load,Clazz.instantialize,Clazz.decorateAsClass,Clazz.floatToInt,Clazz.floatToLong,Clazz.makeConstructor,Clazz.defineEnumConstant,Clazz.exceptionOf,Clazz.newIntArray,Clazz.defineStatics,Clazz.newFloatArray,Clazz.declareType,Clazz.prepareFields,Clazz.superConstructor,Clazz.newByteArray,Clazz.declareInterface,Clazz.p0p,Clazz.pu$h,Clazz.newShortArray,Clazz.innerTypeInstance,Clazz.isClassDefined,Clazz.prepareCallback,Clazz.newArray,Clazz.castNullAs,Clazz.floatToShort,Clazz.superCall,Clazz.decorateAsType,
Clazz.newBooleanArray,Clazz.newCharArray,Clazz.implementOf,Clazz.newDoubleArray,Clazz.overrideConstructor,Clazz.clone,Clazz.doubleToShort,Clazz.getInheritedLevel,Clazz.getParamsType,Clazz.isAF,Clazz.isAB,Clazz.isAI,Clazz.isAS,Clazz.isASS,Clazz.isAP,Clazz.isAFloat,Clazz.isAII,Clazz.isAFF,Clazz.isAFFF,Clazz.tryToSearchAndExecute,Clazz.getStackTrace,Clazz.inheritArgs,Clazz.alert,Clazz.defineMethod,Clazz.overrideMethod,Clazz.declareAnonymous,Clazz.cloneFinals);
